#include "DataType.h"
#include <iterator>
#include <optional>
#include <iostream>

using oint = std::optional<int>;

oint readInt(std::istream &p_in) {
    int i;
    if (p_in >> i) { return {i}; }
    return {};
}

mesh::node::node(int p_node_id, const std::array<double, 3> &p_coors, bool p_vertex)
        : node_id(p_node_id), coors(p_coors), vertex(p_vertex) {}

mesh::node::node(const node &p_node) = default;


std::istream &operator>>(std::istream &p_in, mesh::node &p_node) {
    p_in >> p_node.node_id;
    p_in >> get<0>(p_node.coors)
         >> get<1>(p_node.coors)
         >> get<2>(p_node.coors);
    p_in >> p_node.vertex;
    return p_in;
}

std::ostream &operator<<(std::ostream &p_out, const mesh::node &p_node) {
    p_out << p_node.node_id;
    p_out << get<0>(p_node.coors)
          << get<1>(p_node.coors)
          << get<2>(p_node.coors);
    p_out << p_node.vertex;
    return p_out;
}

std::istream &operator>>(std::istream &p_in, mesh::edge &p_edge) {
    p_in >> p_edge.nodes.first >> p_edge.nodes.second;
    return p_in;
}

std::ostream &operator<<(std::ostream &p_out, mesh::edge &p_edge) {
    p_out << "[" << p_edge.nodes.first << ", " << p_edge.nodes.second << "]";
    return p_out;
}

std::istream &operator>>(std::istream &p_in, mesh::finite_type &p_finite_elem) {
    p_in >> p_finite_elem.element_id
         >> p_finite_elem.material_id;

    oint node_id;
    node_id = readInt(p_in);
    while (node_id) {
        p_finite_elem.node_id.push_back(*node_id);
        node_id = readInt(p_in);
    }
    return p_in;
}

std::ostream &operator<<(std::ostream &p_out, const mesh::finite_type &p_finite_elem) {
    p_out << p_finite_elem.element_id << std::endl
          << p_finite_elem.material_id << std::endl;
    p_out << "[ ";
    for (const auto &elem: p_finite_elem.node_id)
        p_out << elem << " ";
    p_out << "]";
    return p_out;
}

mesh::finite_type::finite_type(int p_element_id, int p_material_id, const std::vector<int> &p_node_id)
        : element_id(p_element_id), material_id(p_material_id), node_id(p_node_id) {}


std::istream &operator>>(std::istream &p_in, mesh::boundary_finite_element &p_bound_finite_elem) {
    p_in >> p_bound_finite_elem.surface_id
         >> p_bound_finite_elem.element_id;

    oint node_id;
    node_id = readInt(p_in);
    while (node_id) {
        p_bound_finite_elem.node_id.push_back(*node_id);
        node_id = readInt(p_in);
    }
    return p_in;
}

std::ostream &operator<<(std::ostream &p_out, const mesh::boundary_finite_element &p_bound_finite_elem) {
    p_out << p_bound_finite_elem.surface_id << std::endl
          << p_bound_finite_elem.element_id << std::endl;
    p_out << "[ ";
    for (const auto &elem: p_bound_finite_elem.node_id)
        p_out << elem << " ";
    p_out << "]";
    return p_out;
}

mesh::boundary_finite_element::boundary_finite_element(int p_element_id, int p_bound_id, const std::vector<int> &p_node_id)
        : element_id(p_element_id), surface_id(p_bound_id), node_id(p_node_id) {}

mesh::edge::edge(int p_id_first_node, int p_id_second_node)
        : nodes(std::make_pair(p_id_first_node, p_id_second_node)) {}

mesh::edge &mesh::edge::operator=(const mesh::edge &p_edge) {
    this->nodes = p_edge.nodes;

    return *this;
}

bool mesh::edge::operator==(const edge &p_edge) const {
    return (nodes.first == p_edge.nodes.first)
           && (nodes.second == p_edge.nodes.second);
}

mesh::edge mesh::edge::create_edge(int p_first_node_id, int p_second_node_id) {
    return {p_first_node_id, p_second_node_id};
}

void mesh::edge::update_mid(int p_middle_node) {
    middle_node = p_middle_node;
}







