﻿#pragma once

#include <tuple>
#include <vector>
#include <unordered_map>
#include <string>
#include <set>
#include <unordered_set>
#include <array>
#include <ranges>
#include <algorithm>
#include <cmath>

namespace mesh {

    namespace ranges = std::ranges;
    namespace rv = std::ranges::views;

    struct edge {
        std::pair<int, int> nodes;
        int middle_node;

        edge() = default;

        edge(int, int);

        edge &operator=(const edge &);

        bool operator==(const edge &) const;

        static edge create_edge(int, int);

        void update_mid(int);

        friend std::istream &operator>>(std::istream &, edge &);

        friend std::ostream &operator<<(std::ostream &, edge &);
    };

    struct node {
        int node_id;
        std::array<double, 3> coors;
        bool vertex = true;

        node() = default;

        node(int, const std::array<double, 3> &, bool);

        node(const node &);

        friend std::istream &operator>>(std::istream &, node &);

        friend std::ostream &operator<<(std::ostream &, const node &);
    };


    struct finite_type {
        int element_id;
        int material_id;
        std::vector<int> node_id;

        finite_type() = default;

        finite_type(int, int, const std::vector<int> &);

        friend std::istream &operator>>(std::istream &, finite_type &);

        friend std::ostream &operator<<(std::ostream &, const finite_type &);
    };

    struct boundary_finite_element {
        int element_id;
        int surface_id;
        std::vector<int> node_id;

        boundary_finite_element() = default;

        boundary_finite_element(int, int, const std::vector<int> &);

        friend std::istream &operator>>(std::istream &, boundary_finite_element &);

        friend std::ostream &operator<<(std::ostream &, const boundary_finite_element &);
    };
}