#include "aneu_mesh_loader.h"
#include "files_not_found.h"
#include <iostream>
#include <fstream>

void mesh::aneu_mesh_loader::load_mesh(const std::string &filename) {
    std::ifstream fin(filename);
    if (!fin) {
        throw files_not_found();
    }

    int number_of_nodes, dimension, number_of_elem, nodes_in_elem, number_of_bound, nodes_in_bound;

    fin >> number_of_nodes >> dimension;
    nodes.reserve(number_of_nodes);
    for (int i = 0; i < number_of_nodes; ++i) {
        node c_node;
        c_node.node_id = i + 1;
        fin >> get<0>(c_node.coors) >> get<1>(c_node.coors) >> get<2>(c_node.coors);
        c_node.vertex = false;
        nodes.push_back(std::move(c_node));
    }

    fin >> number_of_elem >> nodes_in_elem;
    finite_elems.reserve(number_of_elem);
    for (int i = 0; i < number_of_elem; ++i) {
        finite_type c_elem;
        c_elem.element_id = i + 1;
        fin >> c_elem.material_id;
        c_elem.node_id.reserve(nodes_in_elem);
        for (int j = 0; j < nodes_in_elem; ++j) {
            int node_id;
            fin >> node_id;
            c_elem.node_id.push_back(node_id);
        }
        finite_elems.push_back(std::move(c_elem));

    }

    fin >> number_of_bound >> nodes_in_bound;
    boundary.reserve(number_of_bound);
    for (int i = 0; i < number_of_bound; ++i) {
        boundary_finite_element c_bound;
        c_bound.element_id = i + 1;
        fin >> c_bound.surface_id;
        c_bound.node_id.reserve(nodes_in_bound);
        for (int j = 0; j < nodes_in_bound; ++j) {
            int node_id;
            fin >> node_id;
            nodes[node_id - 1].vertex = true;
            c_bound.node_id.push_back(node_id);
        }
        boundary.push_back(std::move(c_bound));

    }
}
