#pragma once

#include "DataType.h"

namespace mesh {

    template<typename T>
    void hashCombine(std::size_t &o_seed, const T &p_val) {
        o_seed ^= std::hash<T>()(p_val) + 0x9e3379b9 + (o_seed << 6) + (o_seed >> 2);
    }

    template<typename T>
    void hashVal(std::size_t &o_seed, const T &p_val) {
        hashCombine(o_seed, p_val);
    }

    template<typename T, typename...Type>
    void hashVal(std::size_t &o_seed, const T &p_val, const Type &...p_args) {
        hashCombine(o_seed, p_val);
        hashVal(o_seed, p_args...);
    }

    template<typename ... Type>
    std::size_t hashVal(const Type &...p_args) {
        std::size_t seed = 0;
        hashVal(seed, p_args...);
        return seed;
    }
}

namespace std {
    template<>
    struct hash<mesh::edge> {
        size_t operator()(const mesh::edge &p_edge) const {
            
            //!!! Данные ребра должны быть отсортированы, в противном случае
            //!!! ребро (1, 2) и (2, 1) имеют разные хеш-значения.
            
            return mesh::hashVal(p_edge.nodes.first, p_edge.nodes.second);
        }
    };
}

