#pragma once

#include "DataType.h"

namespace mesh {

    class mesh_loader {
    public:
        virtual ~mesh_loader() = default;

        virtual void load_mesh(const std::string &) = 0;

    protected:
        std::vector<node> nodes;
        std::vector<finite_type> finite_elems;
        std::vector<boundary_finite_element> boundary;

    public:
        [[nodiscard]] const std::vector<node> &get_node() const;

        [[nodiscard]] const std::vector<finite_type> &get_finite_elems() const;

        [[nodiscard]] const std::vector<boundary_finite_element> &get_boundary() const;

        [[nodiscard]] std::vector<int> get_elems_by_ID(int p_node_id1, int p_node_id2, int p_node_id3) const;

        [[nodiscard]] std::vector<int> get_elems_by_edge(int p_node_id1, int p_node_id2) const;

        [[nodiscard]] std::vector<int> get_elems_by_material(int p_material_id) const;

        [[nodiscard]] std::set<int> get_nodes_by_surface(int p_surface_id) const;

        [[nodiscard]] std::vector<int> get_boundary_by_surface(int p_surface_id) const;

        void insert_new_nodes();

        [[nodiscard]] std::vector<std::set<int>> get_adjacent_nodes() const;

        finite_type get_elems_by_min_ratio();

    private:
        std::array<double, 3> get_middle_of_edge(const edge &);

        node get_middle_node(const edge &);

        node &get_node_by_ID(int);

        std::array<double, 3> get_coors_node_by_ID(int);

        [[nodiscard]] double get_max_base_in_elem(const finite_type &) const;

        [[nodiscard]] double get_min_height_in_elem(const finite_type &) const;
    };
}

